@@ -23,6 +23,7 @@
 #include "weatherforecastmanager.h"
 
 #include <KItinerary/Flight>
+#include <KItinerary/LocationUtil>
 #include <KItinerary/Place>
 #include <KItinerary/Reservation>
 
@@ -65,12 +66,6 @@ private Q_SLOTS:
 
         const auto dt = QDateTime::currentDateTimeUtc();
 
-        TimelineElement group(nullptr, TimelineElement::TripGroup, dt);
-        group.rangeType = TimelineElement::RangeBegin;
-        QTest::newRow("transfer-group-begin") << group << TimelineElement(nullptr, TimelineElement::Transfer, dt);
-        group.rangeType = TimelineElement::RangeEnd;
-        QTest::newRow("transfer-group-end") << TimelineElement(nullptr, TimelineElement::Transfer, dt) << group;
-
         TimelineElement locInfo(nullptr, TimelineElement::LocationInfo, dt);
         TimelineElement hotel(nullptr, TimelineElement::Hotel, dt);
         hotel.rangeType = TimelineElement::RangeBegin;
@@ -105,6 +100,7 @@ private Q_SLOTS:
         TripGroupManager groupMgr;
         groupMgr.setReservationManager(&resMgr);
         groupMgr.setTransferManager(&transferMgr);
+        ctrl->setTripGroupManager(&groupMgr);
 
         TimelineModel model;
         model.setCurrentDateTime({{2024, 3, 31}, {17, 18}});
@@ -121,39 +117,32 @@ private Q_SLOTS:
         QSignalSpy rmSpy(&model, &TimelineModel::rowsRemoved);
         QVERIFY(rmSpy.isValid());
 
-        QCOMPARE(model.rowCount(), 1);
-        QCOMPARE(model.index(0, 0).data(TimelineModel::ElementTypeRole), TimelineElement::TodayMarker);
         ImportController importer;
         importer.setReservationManager(&resMgr);
+        importer.setTripGroupName(u"Test Group"_s);
         importer.importFromUrl(QUrl::fromLocalFile(QLatin1StringView(SOURCE_DIR "/data/boardingpass-v1.pkpass")));
         ctrl->commitImport(&importer);
-        QCOMPARE(insertSpy.size(), 2);
-        QCOMPARE(insertSpy.at(0).at(1).toInt(), 0);
-        QCOMPARE(insertSpy.at(0).at(2).toInt(), 0);
-        QCOMPARE(updateSpy.size(), 1);
-        QCOMPARE(model.rowCount(), 3);
-        QCOMPARE(model.index(0, 0).data(TimelineModel::ElementTypeRole), TimelineElement::Flight);
-        QCOMPARE(model.index(1, 0).data(TimelineModel::ElementTypeRole), TimelineElement::LocationInfo); // DST change
-        QCOMPARE(model.index(2, 0).data(TimelineModel::ElementTypeRole), TimelineElement::TodayMarker);
 
+        QCOMPARE(groupMgr.tripGroups().size(), 1);
+        const auto tgId = groupMgr.tripGroups()[0];
+        model.setTripGroupId(tgId);
+        Test::waitForReset(&model);
+
+        QCOMPARE(model.rowCount(), 1);
+        QCOMPARE(model.index(0, 0).data(TimelineModel::ElementTypeRole), TimelineElement::Flight);
         insertSpy.clear();
         updateSpy.clear();
-        rmSpy.clear();
+
         importer.importFromUrl(QUrl::fromLocalFile(QLatin1StringView(SOURCE_DIR "/data/boardingpass-v2.pkpass")));
+        importer.setTripGroupId(tgId);
         ctrl->commitImport(&importer);
-        QCOMPARE(insertSpy.size(), 1);
+        QCOMPARE(insertSpy.size(), 0);
         QCOMPARE(updateSpy.size(), 1);
         QCOMPARE(updateSpy.at(0).at(0).toModelIndex().row(), 0);
-        QCOMPARE(model.rowCount(), 3);
-
-        insertSpy.clear();
-        updateSpy.clear();
-        rmSpy.clear();
-        Test::clearAll(&resMgr);
-        QCOMPARE(insertSpy.size(), 0);
-        QCOMPARE(updateSpy.size(), 0);
-        QCOMPARE(rmSpy.size(), 2);
         QCOMPARE(model.rowCount(), 1);
+
+        Test::clearAll(&resMgr);
+        QCOMPARE(model.rowCount(), 0);
     }
 
     void testNestedElements()
@@ -168,6 +157,7 @@ private Q_SLOTS:
         TripGroupManager groupMgr;
         groupMgr.setReservationManager(&resMgr);
         groupMgr.setTransferManager(&transferMgr);
+        ctrl->setTripGroupManager(&groupMgr);
 
         TimelineModel model;
         model.setCurrentDateTime({{2024, 3, 29}, {11, 38}});
@@ -185,19 +175,18 @@ private Q_SLOTS:
         QSignalSpy rmSpy(&model, &TimelineModel::rowsRemoved);
         QVERIFY(rmSpy.isValid());
 
-        QCOMPARE(model.rowCount(), 1);
-        QCOMPARE(model.index(0, 0).data(TimelineModel::ElementTypeRole), TimelineElement::TodayMarker);
         ImportController importer;
         importer.setReservationManager(&resMgr);
+        importer.setTripGroupName(u"Test Group"_s);
         importer.importFromUrl(QUrl::fromLocalFile(QLatin1StringView(SOURCE_DIR "/data/haus-randa-v1.json")));
         ctrl->commitImport(&importer);
-        QCOMPARE(insertSpy.size(), 4);
-        QCOMPARE(insertSpy.at(0).at(1).toInt(), 0);
-        QCOMPARE(insertSpy.at(0).at(2).toInt(), 0);
-        QCOMPARE(insertSpy.at(1).at(1).toInt(), 1);
-        QCOMPARE(insertSpy.at(1).at(2).toInt(), 1);
-        QVERIFY(updateSpy.isEmpty());
-        QCOMPARE(model.rowCount(), 5);
+
+        QCOMPARE(groupMgr.tripGroups().size(), 1);
+        const auto tgId = groupMgr.tripGroups()[0];
+        model.setTripGroupId(tgId);
+        Test::waitForReset(&model);
+
+        QCOMPARE(model.rowCount(), 3);
         QCOMPARE(model.index(0, 0).data(TimelineModel::ElementTypeRole), TimelineElement::LocationInfo);
         QCOMPARE(model.index(1, 0).data(TimelineModel::ElementTypeRole), TimelineElement::Hotel);
         QCOMPARE(model.index(1, 0).data(TimelineModel::ElementRangeRole), TimelineElement::RangeBegin);
@@ -209,15 +198,17 @@ private Q_SLOTS:
         updateSpy.clear();
         rmSpy.clear();
         importer.importFromUrl(QUrl::fromLocalFile(QLatin1StringView(SOURCE_DIR "/data/haus-randa-v2.json")));
+        importer.setTripGroupId(tgId);
         ctrl->commitImport(&importer);
-        QCOMPARE(insertSpy.size(), 3);
+
+        QCOMPARE(insertSpy.size(), 2);
         QCOMPARE(updateSpy.size(), 1);
-        QCOMPARE(rmSpy.size(), 3);
+        QCOMPARE(rmSpy.size(), 2);
         QCOMPARE(updateSpy.at(0).at(0).toModelIndex().row(), 1);
         QCOMPARE(insertSpy.at(1).at(1).toInt(), 0);
         QCOMPARE(insertSpy.at(1).at(2).toInt(), 0);
         QCOMPARE(rmSpy.at(0).at(1), 2);
-        QCOMPARE(model.rowCount(), 5);
+        QCOMPARE(model.rowCount(), 3);
 
         // delete a split element
         insertSpy.clear();
@@ -226,9 +217,7 @@ private Q_SLOTS:
         const auto resId = model.data(model.index(1, 0), TimelineModel::BatchIdRole).toString();
         QVERIFY(!resId.isEmpty());
         resMgr.removeReservation(resId);
-        QCOMPARE(rmSpy.size(), 4);
-        QCOMPARE(model.rowCount(), 1);
-        QCOMPARE(model.index(0, 0).data(TimelineModel::ElementTypeRole), TimelineElement::TodayMarker);
+        QCOMPARE(model.rowCount(), 0);
     }
 
     void testCountryInfos()
@@ -242,6 +231,7 @@ private Q_SLOTS:
         TripGroupManager groupMgr;
         groupMgr.setReservationManager(&resMgr);
         groupMgr.setTransferManager(&transferMgr);
+        ctrl->setTripGroupManager(&groupMgr);
 
         TimelineModel model;
         model.setCurrentDateTime({{2024, 3, 29}, {10, 21}}); // after the DST transition in the US!
@@ -250,49 +240,44 @@ private Q_SLOTS:
         model.setReservationManager(&resMgr);
         model.setTransferManager(&transferMgr);
         model.setTripGroupManager(&groupMgr);
-        Test::waitForReset(&model);
-
-        QCOMPARE(model.rowCount(), 1);
-        QCOMPARE(model.index(0, 0).data(TimelineModel::ElementTypeRole), TimelineElement::TodayMarker);
 
         ImportController importer;
         importer.setReservationManager(&resMgr);
         importer.importFromUrl(QUrl::fromLocalFile(QLatin1StringView(SOURCE_DIR "/data/flight-txl-lhr-sfo.json")));
         ctrl->commitImport(&importer);
-        QCOMPARE(model.rowCount(), 8); //  1x begin group, 2x country info, 2x flights, 1x end group, 1x DST info, today marker
 
-        QCOMPARE(model.index(1, 0).data(TimelineModel::ElementTypeRole), TimelineElement::Flight);
-        QCOMPARE(model.index(2, 0).data(TimelineModel::ElementTypeRole), TimelineElement::LocationInfo);
-        auto countryInfo = model.index(2, 0).data(TimelineModel::LocationInformationRole).value<LocationInformation>();
+        QCOMPARE(groupMgr.tripGroups().size(), 1);
+        const auto tgId = groupMgr.tripGroups()[0];
+        model.setTripGroupId(tgId);
+        Test::waitForReset(&model);
+
+        QCOMPARE(model.rowCount(), 4); // 2x country info, 2x flights
+
+        QCOMPARE(model.index(0, 0).data(TimelineModel::ElementTypeRole), TimelineElement::Flight);
+        QCOMPARE(model.index(1, 0).data(TimelineModel::ElementTypeRole), TimelineElement::LocationInfo);
+        auto countryInfo = model.index(1, 0).data(TimelineModel::LocationInformationRole).value<LocationInformation>();
         QCOMPARE(countryInfo.drivingSide(), KItinerary::KnowledgeDb::DrivingSide::Left);
         QCOMPARE(countryInfo.drivingSideDiffers(), true);
         QCOMPARE(countryInfo.powerPlugCompatibility(), LocationInformation::Incompatible);
 
-        QCOMPARE(model.index(3, 0).data(TimelineModel::ElementTypeRole), TimelineElement::Flight);
-        QCOMPARE(model.index(4, 0).data(TimelineModel::ElementTypeRole), TimelineElement::LocationInfo);
-        countryInfo = model.index(4, 0).data(TimelineModel::LocationInformationRole).value<LocationInformation>();
+        QCOMPARE(model.index(2, 0).data(TimelineModel::ElementTypeRole), TimelineElement::Flight);
+        QCOMPARE(model.index(3, 0).data(TimelineModel::ElementTypeRole), TimelineElement::LocationInfo);
+        countryInfo = model.index(3, 0).data(TimelineModel::LocationInformationRole).value<LocationInformation>();
         QCOMPARE(countryInfo.drivingSide(), KItinerary::KnowledgeDb::DrivingSide::Right);
         QCOMPARE(countryInfo.drivingSideDiffers(), false);
         QCOMPARE(countryInfo.powerPlugCompatibility(), LocationInformation::Incompatible);
-        countryInfo = model.index(6, 0).data(TimelineModel::LocationInformationRole).value<LocationInformation>();
-        QCOMPARE(countryInfo.isDst(), true);
-        QCOMPARE(countryInfo.dstDiffers(), true);
-        QCOMPARE(model.index(7, 0).data(TimelineModel::ElementTypeRole), TimelineElement::TodayMarker);
 
         // remove the GB flight should also remove the GB country info
-        auto resId = model.index(1, 0).data(TimelineModel::BatchIdRole).toString();
+        auto resId = model.index(0, 0).data(TimelineModel::BatchIdRole).toString();
         resMgr.removeReservation(resId);
-        QCOMPARE(model.rowCount(), 4);
+        QCOMPARE(model.rowCount(), 2);
         QCOMPARE(model.index(0, 0).data(TimelineModel::ElementTypeRole), TimelineElement::Flight);
         QCOMPARE(model.index(1, 0).data(TimelineModel::ElementTypeRole), TimelineElement::LocationInfo);
-        QCOMPARE(model.index(2, 0).data(TimelineModel::ElementTypeRole), TimelineElement::LocationInfo);
-        QCOMPARE(model.index(3, 0).data(TimelineModel::ElementTypeRole), TimelineElement::TodayMarker);
 
         // remove the US flight should also remove the US country info
         resId = model.index(0, 0).data(TimelineModel::BatchIdRole).toString();
         resMgr.removeReservation(resId);
-        QCOMPARE(model.rowCount(), 1);
-        QCOMPARE(model.index(0, 0).data(TimelineModel::ElementTypeRole), TimelineElement::TodayMarker);
+        QCOMPARE(model.rowCount(), 0);
     }
 
     void testWeatherElements()
@@ -301,166 +286,77 @@ private Q_SLOTS:
 
         ReservationManager resMgr;
         Test::clearAll(&resMgr);
+        auto ctrl = Test::makeAppController();
+        ctrl->setReservationManager(&resMgr);
         TransferManager::clear();
         TransferManager transferMgr;
         TripGroupManager groupMgr;
         groupMgr.setReservationManager(&resMgr);
         groupMgr.setTransferManager(&transferMgr);
+        ctrl->setTripGroupManager(&groupMgr);
         WeatherForecastManager weatherMgr;
         weatherMgr.setTestModeEnabled(true);
 
         TimelineModel model;
         QAbstractItemModelTester tester(&model);
+        model.setHomeCountryIsoCode(QStringLiteral("DE"));
+        model.setCurrentDateTime(QDateTime({2017, 9, 11}, {12, 34}));
         model.setReservationManager(&resMgr);
         model.setTransferManager(&transferMgr);
         model.setTripGroupManager(&groupMgr);
         model.setWeatherForecastManager(&weatherMgr);
+
+        ImportController importer;
+        importer.setReservationManager(&resMgr);
+        importer.importFromUrl(QUrl::fromLocalFile(QLatin1StringView(SOURCE_DIR "/../tests/randa2017.json")));
+        importer.setTripGroupName(u"Randa 2017"_s);
+        ctrl->commitImport(&importer);
+
+        QCOMPARE(groupMgr.tripGroups().size(), 1);
+        model.setTripGroupId(groupMgr.tripGroups()[0]);
         Test::waitForReset(&model);
-        QCOMPARE(model.rowCount(), 1); // no weather data, as we don't know where we are
 
-        // Add an element that will result in a defined location
-        GeoCoordinates geo;
-        geo.setLatitude(52.0f);
-        geo.setLongitude(13.0f);
-        Airport a;
-        a.setName(u"Berlin"_s);
-        a.setGeo(geo);
-        Flight f;
-        f.setArrivalAirport(a);
-        f.setDepartureTime(QDateTime(QDate(2018, 1, 1), QTime(0, 0)));
-        a.setName(u"Zurich"_s);
-        a.setGeo({});
-        f.setDepartureAirport(a);
-        FlightReservation res;
-        res.setReservationFor(f);
-        resMgr.addReservation(res);
-
-        // if there's only <1h left in a day, we wont get a weather element
-        auto currentDate = QDate::currentDate();
-        if (QDateTime::currentDateTimeUtc().time().hour() >= 23) {
-            currentDate = currentDate.addDays(1);
-        }
-
-        QCOMPARE(model.rowCount(), 11); // 1x flight, 1x today, 9x weather
+        QCOMPARE(model.rowCount(), 23);
         QCOMPARE(model.index(0, 0).data(TimelineModel::ElementTypeRole), TimelineElement::Flight);
-        QCOMPARE(model.index(1, 0).data(TimelineModel::ElementTypeRole), TimelineElement::TodayMarker);
-        QCOMPARE(model.index(2, 0).data(TimelineModel::ElementTypeRole), TimelineElement::WeatherForecast);
-        auto fc = model.index(2, 0).data(TimelineModel::WeatherForecastRole).value<WeatherInformation>().forecast;
+        QCOMPARE(model.index(1, 0).data(TimelineModel::ElementTypeRole), TimelineElement::LocationInfo);
+        QCOMPARE(model.index(3, 0).data(TimelineModel::ElementTypeRole), TimelineElement::TrainTrip);
+        QCOMPARE(model.index(6, 0).data(TimelineModel::ElementTypeRole), TimelineElement::TodayMarker);
+        QCOMPARE(model.index(7, 0).data(TimelineModel::ElementTypeRole), TimelineElement::WeatherForecast);
+        auto fc = model.index(7, 0).data(TimelineModel::WeatherForecastRole).value<WeatherInformation>().forecast;
         QVERIFY(fc.isValid());
-        QCOMPARE(fc.dateTime().date(), currentDate);
-        QCOMPARE(fc.minimumTemperature(), 13.0f);
-        QCOMPARE(fc.maximumTemperature(), 52.0f);
-        QCOMPARE(model.index(10, 0).data(TimelineModel::ElementTypeRole), TimelineElement::WeatherForecast);
-        fc = model.index(10, 0).data(TimelineModel::WeatherForecastRole).value<WeatherInformation>().forecast;
-        QVERIFY(fc.isValid());
-        QCOMPARE(fc.dateTime().date(), currentDate.addDays(8));
+        QCOMPARE(fc.dateTime().date(), QDate(2017, 9, 11));
+        QCOMPARE(fc.minimumTemperature(), 7.78326f);
+        QCOMPARE(fc.maximumTemperature(), 46.099f);
 
-        // Add a flight one day from now changing location mid-day
-        geo.setLatitude(46.0f);
-        geo.setLongitude(8.0f);
-        a.setGeo(geo);
-        f.setArrivalAirport(a);
-        f.setDepartureTime(QDateTime(currentDate.addDays(1), QTime(12, 0)));
-        f.setArrivalTime(QDateTime(currentDate.addDays(1), QTime(14, 0)));
-        res.setReservationFor(f);
-        resMgr.addReservation(res);
+        QCOMPARE(model.index(16, 0).data(TimelineModel::ElementTypeRole), TimelineElement::WeatherForecast);
+        fc = model.index(16, 0).data(TimelineModel::WeatherForecastRole).value<WeatherInformation>().forecast;
+        QVERIFY(fc.isValid());
+        QCOMPARE(fc.dateTime().date(), QDate(2017, 9, 15));
+        QCOMPARE(fc.minimumTemperature(), 7.78326f);
+        QCOMPARE(fc.maximumTemperature(), 46.099f);
 
-        QCOMPARE(model.rowCount(), 13); // 2x flight, 1x today, 10x weather
-        QCOMPARE(model.index(0, 0).data(TimelineModel::ElementTypeRole), TimelineElement::Flight);
-        QCOMPARE(model.index(1, 0).data(TimelineModel::ElementTypeRole), TimelineElement::TodayMarker);
-        QCOMPARE(model.index(2, 0).data(TimelineModel::ElementTypeRole), TimelineElement::WeatherForecast);
-        fc = model.index(2, 0).data(TimelineModel::WeatherForecastRole).value<WeatherInformation>().forecast;
+        QCOMPARE(model.index(19, 0).data(TimelineModel::ElementTypeRole), TimelineElement::TrainTrip);
+        QCOMPARE(model.index(20, 0).data(TimelineModel::ElementTypeRole), TimelineElement::WeatherForecast);
+        fc = model.index(20, 0).data(TimelineModel::WeatherForecastRole).value<WeatherInformation>().forecast;
         QVERIFY(fc.isValid());
-        QCOMPARE(fc.dateTime().date(), currentDate);
-        QCOMPARE(model.index(3, 0).data(TimelineModel::ElementTypeRole), TimelineElement::WeatherForecast);
-        fc = model.index(3, 0).data(TimelineModel::WeatherForecastRole).value<WeatherInformation>().forecast;
+        QCOMPARE(fc.dateTime().date(), QDate(2017, 9, 15));
+        QCOMPARE(fc.minimumTemperature(), 8.56222f);
+        QCOMPARE(fc.maximumTemperature(), 47.4503f);
+
+        QCOMPARE(model.index(21, 0).data(TimelineModel::ElementTypeRole), TimelineElement::Flight);
+        QCOMPARE(model.index(22, 0).data(TimelineModel::ElementTypeRole), TimelineElement::WeatherForecast);
+        fc = model.index(22, 0).data(TimelineModel::WeatherForecastRole).value<WeatherInformation>().forecast;
         QVERIFY(fc.isValid());
-        QCOMPARE(fc.minimumTemperature(), 13.0f);
-        QCOMPARE(fc.maximumTemperature(), 52.0f);
-        QCOMPARE(fc.dateTime(), QDateTime(currentDate.addDays(1), QTime(0, 0)));
-        QCOMPARE(model.index(4, 0).data(TimelineModel::ElementTypeRole), TimelineElement::Flight);
-        QCOMPARE(model.index(5, 0).data(TimelineModel::ElementTypeRole), TimelineElement::WeatherForecast);
-        fc = model.index(5, 0).data(TimelineModel::WeatherForecastRole).value<WeatherInformation>().forecast;
-        QVERIFY(fc.isValid());
-        QCOMPARE(fc.minimumTemperature(), 8.0f);
-        QCOMPARE(fc.maximumTemperature(), 46.0f);
-        QCOMPARE(fc.dateTime(), QDateTime(currentDate.addDays(1), QTime(14, 0)));
-        QCOMPARE(model.index(6, 0).data(TimelineModel::ElementTypeRole), TimelineElement::WeatherForecast);
-        fc = model.index(6, 0).data(TimelineModel::WeatherForecastRole).value<WeatherInformation>().forecast;
-        QCOMPARE(fc.minimumTemperature(), 8.0f);
-        QCOMPARE(fc.maximumTemperature(), 46.0f);
-        QVERIFY(fc.isValid());
-        QCOMPARE(fc.dateTime(), QDateTime(currentDate.addDays(2), QTime(0, 0)));
+        QCOMPARE(fc.dateTime().date(), QDate(2017, 9, 15));
+        QCOMPARE(fc.minimumTemperature(), 13.2878f);
+        QCOMPARE(fc.maximumTemperature(), 52.5597f);
 
         // check we get update signals for all weather elements
         QSignalSpy spy(&model, &TimelineModel::dataChanged);
         QVERIFY(spy.isValid());
         Q_EMIT weatherMgr.forecastUpdated();
-        QCOMPARE(spy.size(), 10);
-        QCOMPARE(model.rowCount(), 13); // 2x flight, 1x today, 10x weather
-
-        fc = model.index(3, 0).data(TimelineModel::WeatherForecastRole).value<WeatherInformation>().forecast;
-        QVERIFY(fc.isValid());
-        QCOMPARE(fc.minimumTemperature(), 13.0f);
-        QCOMPARE(fc.maximumTemperature(), 52.0f);
-        QCOMPARE(fc.dateTime(), QDateTime(currentDate.addDays(1), QTime(0, 0)));
-        fc = model.index(9, 0).data(TimelineModel::WeatherForecastRole).value<WeatherInformation>().forecast;
-        QVERIFY(fc.isValid());
-        QCOMPARE(fc.minimumTemperature(), 8.0f);
-        QCOMPARE(fc.maximumTemperature(), 46.0f);
-        QCOMPARE(fc.dateTime(), QDateTime(currentDate.addDays(5), QTime(0, 0)));
-
-        // add a location change far in the future, this must not change anything
-        // NOTE: this adds visible trip grouping
-        geo.setLatitude(60.0f);
-        geo.setLongitude(11.0f);
-        a.setGeo(geo);
-        f.setArrivalAirport(a);
-        f.setDepartureTime(QDateTime(currentDate.addYears(1), QTime(6, 0)));
-        res.setReservationFor(f);
-        resMgr.addReservation(res);
-        QCOMPARE(model.rowCount(), 16);
-
-        fc = model.index(3, 0).data(TimelineModel::WeatherForecastRole).value<WeatherInformation>().forecast;
-        QVERIFY(fc.isValid());
-        QCOMPARE(fc.minimumTemperature(), 13.0f);
-        QCOMPARE(fc.maximumTemperature(), 52.0f);
-        QCOMPARE(fc.dateTime(), QDateTime(currentDate.addDays(1), QTime(0, 0)));
-        fc = model.index(11, 0).data(TimelineModel::WeatherForecastRole).value<WeatherInformation>().forecast;
-        QVERIFY(fc.isValid());
-        QCOMPARE(fc.minimumTemperature(), 8.0f);
-        QCOMPARE(fc.maximumTemperature(), 46.0f);
-        QCOMPARE(fc.dateTime(), QDateTime(currentDate.addDays(5), QTime(0, 0)));
-
-        // result is the same when data hasn't been added incrementally
-        {
-            TimelineModel model;
-            QAbstractItemModelTester tester(&model);
-            model.setReservationManager(&resMgr);
-            model.setTransferManager(&transferMgr);
-            model.setTripGroupManager(&groupMgr);
-            model.setWeatherForecastManager(&weatherMgr);
-            Test::waitForReset(&model);
-            QCOMPARE(model.rowCount(), 16);
-
-            fc = model.index(3, 0).data(TimelineModel::WeatherForecastRole).value<WeatherInformation>().forecast;
-            QVERIFY(fc.isValid());
-            QCOMPARE(fc.minimumTemperature(), 13.0f);
-            QCOMPARE(fc.maximumTemperature(), 52.0f);
-            QCOMPARE(fc.dateTime(), QDateTime(currentDate.addDays(1), QTime(0, 0)));
-            fc = model.index(11, 0).data(TimelineModel::WeatherForecastRole).value<WeatherInformation>().forecast;
-            QVERIFY(fc.isValid());
-            QCOMPARE(fc.minimumTemperature(), 8.0f);
-            QCOMPARE(fc.maximumTemperature(), 46.0f);
-            QCOMPARE(fc.dateTime(), QDateTime(currentDate.addDays(5), QTime(0, 0)));
-        }
-
-        // clean up
-        auto resId = model.index(15, 0).data(TimelineModel::BatchIdRole).toString();
-        resMgr.removeReservation(resId);
-        resId = model.index(5, 0).data(TimelineModel::BatchIdRole).toString();
-        resMgr.removeReservation(resId);
-        QCOMPARE(model.rowCount(), 13);
+        QCOMPARE(model.rowCount(), 23);
+        QCOMPARE(spy.size(), 9);
 
         // test case: two consecutive location changes, the first one to an unknown location
         // result: the weather element before the first location change ends with the start of that
@@ -498,8 +394,13 @@ private Q_SLOTS:
         importer.setReservationManager(&resMgr);
         importer.importFromUrl(QUrl::fromLocalFile(QLatin1StringView(SOURCE_DIR "/data/weather-no-location-change.json")));
         ctrl->commitImport(&importer);
+
+        QCOMPARE(groupMgr.tripGroups().size(), 1);
+        model.setTripGroupId(groupMgr.tripGroups()[0]);
+        Test::waitForReset(&model);
+
         ModelVerificationPoint vp0(QLatin1StringView(SOURCE_DIR "/data/weather-no-location-change.model"));
-        vp0.setRoleFilter({TimelineModel::BatchIdRole, TimelineModel::TripGroupIdRole});
+        vp0.setRoleFilter({TimelineModel::BatchIdRole});
         vp0.setJsonPropertyFilter({"elements"_L1});
         QVERIFY(vp0.verify(&model));
     }
@@ -517,6 +418,7 @@ private Q_SLOTS:
         TripGroupManager groupMgr;
         groupMgr.setReservationManager(&resMgr);
         groupMgr.setTransferManager(&transferMgr);
+        ctrl->setTripGroupManager(&groupMgr);
 
         TimelineModel model;
         model.setCurrentDateTime({{2024, 3, 29}, {10, 21}}); // after the DST transition in the US!
@@ -524,8 +426,6 @@ private Q_SLOTS:
         model.setReservationManager(&resMgr);
         model.setTransferManager(&transferMgr);
         model.setTripGroupManager(&groupMgr);
-        Test::waitForReset(&model);
-        QCOMPARE(model.rowCount(), 1); // 1x TodayMarker
 
         QSignalSpy insertSpy(&model, &TimelineModel::rowsInserted);
         QVERIFY(insertSpy.isValid());
@@ -534,20 +434,25 @@ private Q_SLOTS:
         QSignalSpy rmSpy(&model, &TimelineModel::rowsRemoved);
         QVERIFY(rmSpy.isValid());
 
+        const auto tgId = groupMgr.createEmptyGroup(u"Test Trip"_s);
+        QVERIFY(!tgId.isEmpty());
+        QCOMPARE(groupMgr.tripGroups().size(), 1);
+        model.setTripGroupId(tgId);
+        Test::waitForReset(&model);
+
         // full import at runtime
         ImportController importer;
         importer.setReservationManager(&resMgr);
         importer.importFromUrl(QUrl::fromLocalFile(QLatin1StringView(SOURCE_DIR "/data/google-multi-passenger-flight.json")));
+        importer.setTripGroupId(tgId);
         ctrl->commitImport(&importer);
-        QCOMPARE(model.rowCount(), 7); // 1x group begin, 2x Flight, 1x tz change info, 1x group end, 1x DST info, 1x TodayMarker
-        QCOMPARE(updateSpy.count(), 4);
-        QCOMPARE(rmSpy.count(), 3);
-        QCOMPARE(model.index(1, 0).data(TimelineModel::ElementTypeRole), TimelineElement::Flight);
+
+        QCOMPARE(model.rowCount(), 4);
+        QCOMPARE(model.index(0, 0).data(TimelineModel::ElementTypeRole), TimelineElement::Flight);
+        QCOMPARE(model.index(1, 0).data(TimelineModel::ElementTypeRole), TimelineElement::LocationInfo);
         QCOMPARE(model.index(2, 0).data(TimelineModel::ElementTypeRole), TimelineElement::Flight);
         QCOMPARE(model.index(3, 0).data(TimelineModel::ElementTypeRole), TimelineElement::LocationInfo);
-        QCOMPARE(model.index(5, 0).data(TimelineModel::ElementTypeRole), TimelineElement::LocationInfo);
-        QCOMPARE(model.index(6, 0).data(TimelineModel::ElementTypeRole), TimelineElement::TodayMarker);
-        QCOMPARE(resMgr.reservationsForBatch(model.index(1, 0).data(TimelineModel::BatchIdRole).toString()).size(), 2);
+        QCOMPARE(resMgr.reservationsForBatch(model.index(0, 0).data(TimelineModel::BatchIdRole).toString()).size(), 2);
         QCOMPARE(resMgr.reservationsForBatch(model.index(2, 0).data(TimelineModel::BatchIdRole).toString()).size(), 2);
 
         // already existing data
@@ -558,33 +463,40 @@ private Q_SLOTS:
             model.setReservationManager(&resMgr);
             model.setTransferManager(&transferMgr);
             model.setTripGroupManager(&groupMgr);
+            model.setTripGroupId(tgId);
             Test::waitForReset(&model);
 
-            QCOMPARE(model.rowCount(), 7); // 1x group begin, 2x Flight, 1x tz change info, 1x group end, 1x DST info, 1x TodayMarker
-            QCOMPARE(model.index(1, 0).data(TimelineModel::ElementTypeRole), TimelineElement::Flight);
+            QCOMPARE(model.rowCount(), 4); // 2x Flight, 2x tz change info
+            QCOMPARE(model.index(0, 0).data(TimelineModel::ElementTypeRole), TimelineElement::Flight);
+            QCOMPARE(model.index(1, 0).data(TimelineModel::ElementTypeRole), TimelineElement::LocationInfo);
             QCOMPARE(model.index(2, 0).data(TimelineModel::ElementTypeRole), TimelineElement::Flight);
             QCOMPARE(model.index(3, 0).data(TimelineModel::ElementTypeRole), TimelineElement::LocationInfo);
-            QCOMPARE(model.index(5, 0).data(TimelineModel::ElementTypeRole), TimelineElement::LocationInfo);
-            QCOMPARE(model.index(6, 0).data(TimelineModel::ElementTypeRole), TimelineElement::TodayMarker);
-            QCOMPARE(resMgr.reservationsForBatch(model.index(1, 0).data(TimelineModel::BatchIdRole).toString()).size(), 2);
+            QCOMPARE(resMgr.reservationsForBatch(model.index(0, 0).data(TimelineModel::BatchIdRole).toString()).size(), 2);
             QCOMPARE(resMgr.reservationsForBatch(model.index(2, 0).data(TimelineModel::BatchIdRole).toString()).size(), 2);
         }
 
+        groupMgr.suspend(); // avoid auto-regrouping while we mess with things here
+
         // update splits element
         updateSpy.clear();
         insertSpy.clear();
         rmSpy.clear();
-        auto resId = model.index(1, 0).data(TimelineModel::BatchIdRole).toString();
+        const auto resId = model.index(0, 0).data(TimelineModel::BatchIdRole).toString();
         QVERIFY(!resId.isEmpty());
+        const auto resIds = resMgr.reservationsForBatch(resId);
         auto res = resMgr.reservation(resId).value<FlightReservation>();
         auto flight = res.reservationFor().value<Flight>();
         flight.setDepartureTime(flight.departureTime().addDays(1));
         res.setReservationFor(flight);
         resMgr.updateReservation(resId, res);
-        QCOMPARE(model.rowCount(), 8);
+        groupMgr.addToGroup(resIds, tgId); // manually group, as auto-grouping would reject the trip we just created here
+
+        QCOMPARE(model.rowCount(), 5);
+        QCOMPARE(model.index(0, 0).data(TimelineModel::ElementTypeRole), TimelineElement::Flight);
+        QCOMPARE(model.index(1, 0).data(TimelineModel::ElementTypeRole), TimelineElement::Flight);
         QCOMPARE(updateSpy.count(), 4);
-        QCOMPARE(insertSpy.count(), 6);
-        QCOMPARE(rmSpy.count(), 5);
+        QCOMPARE(insertSpy.count(), 3);
+        QCOMPARE(rmSpy.count(), 2);
 
         // update merges two elements
         updateSpy.clear();
@@ -593,18 +505,20 @@ private Q_SLOTS:
         flight.setDepartureTime(flight.departureTime().addDays(-1));
         res.setReservationFor(flight);
         resMgr.updateReservation(resId, res);
-        QCOMPARE(model.rowCount(), 5);
-        QCOMPARE(updateSpy.count(), 3);
-        QCOMPARE(rmSpy.count(), 5);
-        QCOMPARE(insertSpy.count(), 2);
+        QCOMPARE(model.rowCount(), 4);
+        QCOMPARE(model.index(0, 0).data(TimelineModel::ElementTypeRole), TimelineElement::Flight);
+        QCOMPARE(model.index(1, 0).data(TimelineModel::ElementTypeRole), TimelineElement::LocationInfo);
+        QCOMPARE(resMgr.reservationsForBatch(model.index(0, 0).data(TimelineModel::BatchIdRole).toString()).size(), 2);
+
+        QCOMPARE(updateSpy.count(), 5);
+        QCOMPARE(rmSpy.count(), 1);
+        QCOMPARE(insertSpy.count(), 0);
 
         // removal of merged items
         updateSpy.clear();
         rmSpy.clear();
         Test::clearAll(&resMgr);
-        QCOMPARE(model.rowCount(), 1);
-        QCOMPARE(rmSpy.count(), 7);
-        QCOMPARE(updateSpy.count(), 3);
+        QCOMPARE(model.rowCount(), 0);
     }
 
     void testDayChange()
@@ -618,45 +532,45 @@ private Q_SLOTS:
         TripGroupManager groupMgr;
         groupMgr.setReservationManager(&resMgr);
         groupMgr.setTransferManager(&transferMgr);
+        ctrl->setTripGroupManager(&groupMgr);
         WeatherForecastManager weatherMgr;
         weatherMgr.setTestModeEnabled(true);
 
+        ImportController importer;
+        importer.setReservationManager(&resMgr);
+        importer.importFromUrl(QUrl::fromLocalFile(QLatin1StringView(SOURCE_DIR "/../tests/randa2017.json")));
+        importer.setTripGroupName(u"Randa 2017"_s);
+        ctrl->commitImport(&importer);
+        QCOMPARE(groupMgr.tripGroups().size(), 1);
+
         TimelineModel model;
         QAbstractItemModelTester tester(&model);
         model.setHomeCountryIsoCode(QStringLiteral("DE"));
-        model.setCurrentDateTime(QDateTime({2196, 10, 14}, {12, 34}));
+        model.setCurrentDateTime(QDateTime({2017, 9, 1}, {12, 34}));
         model.setReservationManager(&resMgr);
         model.setTransferManager(&transferMgr);
         model.setTripGroupManager(&groupMgr);
         model.setWeatherForecastManager(&weatherMgr);
+        model.setTripGroupId(groupMgr.tripGroups()[0]);
         Test::waitForReset(&model);
 
+        // before the trip, close enough for weather but no today marker
         ModelVerificationPoint vp0(QLatin1StringView(SOURCE_DIR "/data/timeline/daychange-r0.model"));
         vp0.setRoleFilter({TimelineModel::BatchIdRole});
         vp0.setJsonPropertyFilter({"elements"_L1});
         QVERIFY(vp0.verify(&model));
 
         // changing the day should move the today marker
-        model.setCurrentDateTime(QDateTime({2196, 10, 15}, {0, 15}));
+        model.setCurrentDateTime(QDateTime({2017, 9, 11}, {0, 15}));
         ModelVerificationPoint vp1(QLatin1StringView(SOURCE_DIR "/data/timeline/daychange-r1.model"));
         vp1.setRoleFilter({TimelineModel::BatchIdRole});
         vp1.setJsonPropertyFilter({"elements"_L1});
         QVERIFY(vp1.verify(&model));
 
-        // load something to define the current location, so we get weather
-        ImportController importer;
-        importer.setReservationManager(&resMgr);
-        importer.importFromUrl(QUrl::fromLocalFile(QLatin1StringView(SOURCE_DIR "/data/flight-txl-lhr-sfo.json")));
-        ctrl->commitImport(&importer);
-        ModelVerificationPoint vp2(QLatin1StringView(SOURCE_DIR "/data/timeline/daychange-r2.model"));
-        vp2.setRoleFilter({TimelineModel::BatchIdRole, TimelineModel::TripGroupIdRole});
-        vp2.setJsonPropertyFilter({"elements"_L1});
-        QVERIFY(vp2.verify(&model));
-
         // changing the day should move the today marker and weather one day forward
-        model.setCurrentDateTime(QDateTime({2196, 10, 16}, {19, 30}));
+        model.setCurrentDateTime(QDateTime({2017, 9, 12}, {19, 30}));
         ModelVerificationPoint vp3(QLatin1StringView(SOURCE_DIR "/data/timeline/daychange-r3.model"));
-        vp3.setRoleFilter({TimelineModel::BatchIdRole, TimelineModel::TripGroupIdRole});
+        vp3.setRoleFilter({TimelineModel::BatchIdRole});
         vp3.setJsonPropertyFilter({"elements"_L1});
         QVERIFY(vp3.verify(&model));
     }
@@ -668,6 +582,9 @@ private Q_SLOTS:
         QDirIterator it(QLatin1StringView(SOURCE_DIR "/data/timeline/"), {QLatin1StringView("*.json")});
         while (it.hasNext()) {
             it.next();
+            if (it.fileInfo().fileName().endsWith(".context.json"_L1)) {
+                continue;
+            }
             const auto baseName = it.fileInfo().baseName();
             QTest::newRow(baseName.toUtf8().constData()) << baseName;
         }
@@ -675,15 +592,14 @@ private Q_SLOTS:
 
     void testContent()
     {
+        if constexpr (sizeof(double) == 4) {
+            QSKIP("test data assumes 64bit double resolution - skipping");
+        }
         QFETCH(QString, baseName);
         ReservationManager resMgr;
         Test::clearAll(&resMgr);
         auto ctrl = Test::makeAppController();
         ctrl->setReservationManager(&resMgr);
-        ImportController importer;
-        importer.setReservationManager(&resMgr);
-        importer.importFromUrl(QUrl::fromLocalFile(QLatin1StringView(SOURCE_DIR "/data/timeline/") + baseName + QLatin1StringView(".json")));
-        ctrl->commitImport(&importer);
         WeatherForecastManager weatherMgr;
         weatherMgr.setTestModeEnabled(true);
 
@@ -710,6 +626,24 @@ private Q_SLOTS:
         TripGroupManager groupMgr;
         groupMgr.setReservationManager(&resMgr);
         groupMgr.setTransferManager(&transferMgr);
+        ctrl->setTripGroupManager(&groupMgr);
+
+        ImportController importer;
+        importer.setReservationManager(&resMgr);
+        importer.importFromUrl(QUrl::fromLocalFile(QLatin1StringView(SOURCE_DIR "/data/timeline/") + baseName + ".context.json"_L1));
+        ctrl->commitImport(&importer);
+
+        QVariant initialLocation;
+        if (!groupMgr.tripGroups().empty()) {
+            const auto tg = groupMgr.tripGroup(groupMgr.tripGroups()[0]);
+            QCOMPARE(tg.elements().size(), 1);
+            initialLocation = KItinerary::LocationUtil::arrivalLocation(resMgr.reservation(tg.elements()[0]));
+        }
+
+        QSignalSpy tgAddSpy(&groupMgr, &TripGroupManager::tripGroupAdded);
+        importer.importFromUrl(QUrl::fromLocalFile(QLatin1StringView(SOURCE_DIR "/data/timeline/") + baseName + ".json"_L1));
+        ctrl->commitImport(&importer);
+        QCOMPARE(tgAddSpy.size(), 1);
 
         TimelineModel model;
         QAbstractItemModelTester tester(&model);
@@ -719,18 +653,32 @@ private Q_SLOTS:
         model.setTripGroupManager(&groupMgr);
         model.setWeatherForecastManager(&weatherMgr);
         model.setTransferManager(&transferMgr);
+        model.setTripGroupId(tgAddSpy.at(0).at(0).toString());
+        model.setProperty("initialLocation", initialLocation);
         Test::waitForReset(&model);
 
         // check state is correct for data imported at the start
         ModelVerificationPoint vp(QLatin1StringView(SOURCE_DIR "/data/timeline/") + baseName + QLatin1StringView(".model"));
-        vp.setRoleFilter({TimelineModel::BatchIdRole, TimelineModel::TripGroupIdRole});
+        vp.setRoleFilter({TimelineModel::BatchIdRole});
         vp.setJsonPropertyFilter({"reservationId"_L1, "elements"_L1});
         QVERIFY(vp.verify(&model));
 
         // retry with loading during runtime
         Test::clearAll(&resMgr);
-        importer.importFromUrl(QUrl::fromLocalFile(QLatin1StringView(SOURCE_DIR "/data/timeline/") + baseName + QLatin1StringView(".json")));
+        importer.importFromUrl(QUrl::fromLocalFile(QLatin1StringView(SOURCE_DIR "/data/timeline/") + baseName + ".context.json"_L1));
+        importer.setTripGroupName(u"Context Group"_s);
         ctrl->commitImport(&importer);
+        tgAddSpy.clear();
+
+        const auto newTgId = groupMgr.createEmptyGroup(u"Test Group"_s);
+        importer.setTripGroupId(newTgId);
+        importer.importFromUrl(QUrl::fromLocalFile(QLatin1StringView(SOURCE_DIR "/data/timeline/") + baseName + ".json"_L1));
+        ctrl->commitImport(&importer);
+        QCOMPARE(tgAddSpy.size(), 1);
+        QCOMPARE(tgAddSpy.at(0).at(0).toString(), newTgId);
+
+        model.setTripGroupId(newTgId);
+        Test::waitForReset(&model);
         QVERIFY(vp.verify(&model));
     }
 
@@ -771,7 +719,7 @@ private Q_SLOTS:
         model.setTripGroupId(tgId1);
 
         Test::waitForReset(&model);
-        QCOMPARE(model.rowCount(), 3);
+        QCOMPARE(model.rowCount(), 4);
 
         model.setTripGroupId(tgId2);
         QCOMPARE(model.rowCount(), 0);
