@@ -47,6 +47,10 @@ static int copySettings(const QSettings *from, QSettings *to)
 Exporter::Exporter(KItinerary::File *file)
     : m_file(file)
 {
+    QJsonObject versionData{
+        {"formatVersion"_L1, BUNDLE_FORMAT_VERSION}
+    };
+    m_file->addCustomData(BUNDLE_VERSION_DOMAIN, "version"_L1, QJsonDocument(versionData).toJson(QJsonDocument::Compact));
 }
 
 void Exporter::exportReservations(const ReservationManager *resMgr)
@@ -170,6 +174,11 @@ void Exporter::exportPasses(const PassManager *passMgr)
     }
 }
 
+void Exporter::exportPass(const QString &passId, const PassManager *passMgr)
+{
+    m_file->addCustomData(BUNDLE_PASS_DOMAIN, passId, passMgr->passData(passId));
+}
+
 void Exporter::exportHealthCertificates(const HealthCertificateManager *healthCertMgr)
 {
     for (int i = 0; i < healthCertMgr->rowCount(); ++i) {
@@ -221,6 +230,12 @@ Importer::Importer(const KItinerary::File *file)
 {
 }
 
+int Importer::formatVersion() const
+{
+    const auto versionData = QJsonDocument::fromJson(m_file->customData(BUNDLE_VERSION_DOMAIN, "version"_L1)).object();
+    return versionData.value("formatVersion"_L1).toInt(0);
+}
+
 int Importer::importReservations(ReservationManager *resMgr)
 {
     const auto resIds = m_file->reservations();
