@@ -38,6 +38,7 @@
 
 #include <QJSEngine>
 #include <QJSValue>
+#include <QMetaProperty>
 
 #include <QCoreApplication>
 #include <QDateTime>
@@ -85,8 +86,6 @@ void TimelineDelegateController::setReservationManager(ReservationManager *resMg
     m_resMgr = resMgr;
     Q_EMIT setupChanged();
     Q_EMIT contentChanged();
-    Q_EMIT departureChanged();
-    Q_EMIT arrivalChanged();
     Q_EMIT journeyChanged();
     Q_EMIT previousLocationChanged();
     Q_EMIT layoutChanged();
@@ -113,30 +112,14 @@ void TimelineDelegateController::setLiveDataManager(LiveDataManager *liveDataMgr
 
     m_liveDataMgr = liveDataMgr;
     Q_EMIT setupChanged();
-    Q_EMIT departureChanged();
-    Q_EMIT arrivalChanged();
     Q_EMIT journeyChanged();
     Q_EMIT layoutChanged();
 
-    connect(m_liveDataMgr, &LiveDataManager::arrivalUpdated, this, &TimelineDelegateController::checkForUpdate);
-    connect(m_liveDataMgr, &LiveDataManager::departureUpdated, this, &TimelineDelegateController::checkForUpdate);
-    connect(m_liveDataMgr, &LiveDataManager::arrivalUpdated, this, [this](const auto &batchId) {
-        if (batchId == m_batchId) {
-            Q_EMIT arrivalChanged();
-            Q_EMIT journeyChanged();
-        }
-    });
-    connect(m_liveDataMgr, &LiveDataManager::departureUpdated, this, [this](const auto &batchId) {
-        if (batchId == m_batchId) {
-            Q_EMIT departureChanged();
-            Q_EMIT journeyChanged();
-        }
-    });
+    connect(m_liveDataMgr, &LiveDataManager::journeyUpdated, this, &TimelineDelegateController::checkForUpdate);
     connect(m_liveDataMgr, &LiveDataManager::journeyUpdated, this, [this](const auto &batchId) {
         if (batchId == m_batchId) {
-            Q_EMIT departureChanged();
-            Q_EMIT arrivalChanged();
             Q_EMIT journeyChanged();
+            Q_EMIT layoutChanged();
         }
     });
 
@@ -188,8 +171,6 @@ void TimelineDelegateController::setBatchId(const QString &batchId)
     m_batchId = batchId;
     Q_EMIT batchIdChanged();
     Q_EMIT contentChanged();
-    Q_EMIT departureChanged();
-    Q_EMIT arrivalChanged();
     Q_EMIT journeyChanged();
     Q_EMIT previousLocationChanged();
     checkForUpdate(batchId);
@@ -276,6 +257,30 @@ KPublicTransport::JourneySection TimelineDelegateController::journey() const
     return m_liveDataMgr->journey(m_batchId);
 }
 
+KPublicTransport::JourneySection TimelineDelegateController::trip() const
+{
+    if (!m_liveDataMgr || m_batchId.isEmpty()) {
+        return {};
+    }
+    return m_liveDataMgr->trip(m_batchId);
+}
+
+qsizetype TimelineDelegateController::tripDepartureIndex() const
+{
+    if (!m_liveDataMgr || m_batchId.isEmpty()) {
+        return {};
+    }
+    return m_liveDataMgr->tripDepartureIndex(m_batchId);
+}
+
+qsizetype TimelineDelegateController::tripArrivalIndex() const
+{
+    if (!m_liveDataMgr || m_batchId.isEmpty()) {
+        return {};
+    }
+    return m_liveDataMgr->tripArrivalIndex(m_batchId);
+}
+
 void TimelineDelegateController::checkForUpdate(const QString &batchId)
 {
     if (!m_resMgr || m_batchId.isEmpty()) {
@@ -360,8 +365,6 @@ void TimelineDelegateController::batchChanged(const QString &batchId)
     }
     checkForUpdate(batchId);
     Q_EMIT contentChanged();
-    Q_EMIT arrivalChanged();
-    Q_EMIT departureChanged();
     Q_EMIT journeyChanged();
     Q_EMIT previousLocationChanged();
 }
@@ -432,9 +435,19 @@ bool TimelineDelegateController::isPublicTransport() const
 
 static bool isJourneyCandidate(const QVariant &res)
 {
-    // TODO do we really need to constrain this to trains/buses? a long distance train can be a suitable alternative for a missed short distance flight for
-    // example
-    return LocationUtil::isLocationChange(res) && (JsonLd::isA<TrainReservation>(res) || JsonLd::isA<BusReservation>(res));
+    if (JsonLd::isA<TrainReservation>(res) || JsonLd::isA<BusReservation>(res)) {
+        return true;
+    }
+
+    // any other means of transport: if the distance is realistic for long distance trains
+    if (!LocationUtil::isLocationChange(res)) {
+        return false;
+    }
+
+    const auto from = LocationUtil::geo(LocationUtil::departureLocation(res));
+    const auto to = LocationUtil::geo(LocationUtil::arrivalLocation(res));
+    const auto dist = LocationUtil::distance(from, to);
+    return !std::isnan(dist) && dist < 2'000'000;
 }
 
 static bool isLayover(const QVariant &res1, const QVariant &res2)
@@ -454,7 +467,7 @@ static bool isLayover(const QVariant &res1, const QVariant &res2)
     return LocationUtil::isSameLocation(LocationUtil::arrivalLocation(res1), LocationUtil::departureLocation(res2), LocationUtil::WalkingDistance);
 }
 
-KPublicTransport::JourneyRequest TimelineDelegateController::journeyRequestOne() const
+KPublicTransport::JourneyRequest TimelineDelegateController::journeyRequest() const
 {
     if (!m_resMgr || m_batchId.isEmpty() || !m_liveDataMgr) {
         return {};
@@ -467,7 +480,6 @@ KPublicTransport::JourneyRequest TimelineDelegateController::journeyRequestOne()
 
     KPublicTransport::JourneyRequest req;
     req.setFrom(PublicTransport::locationFromPlace(LocationUtil::departureLocation(res), res));
-    req.setTo(PublicTransport::locationFromPlace(LocationUtil::arrivalLocation(res), res));
     req.setDateTime(std::max(QDateTime::currentDateTime(), SortUtil::startDateTime(res)));
     req.setDateTimeMode(KPublicTransport::JourneyRequest::Departure);
     req.setDownloadAssets(true);
@@ -477,16 +489,20 @@ KPublicTransport::JourneyRequest TimelineDelegateController::journeyRequestOne()
     return req;
 }
 
-KPublicTransport::JourneyRequest TimelineDelegateController::journeyRequestFull() const
+QList<KPublicTransport::Location> TimelineDelegateController::journeyDestinations() const
 {
-    auto req = journeyRequestOne();
-    if (!req.isValid()) {
+    if (!m_resMgr || m_batchId.isEmpty() || !m_liveDataMgr) {
         return {};
     }
 
-    // find full journey by looking at subsequent elements
+    QList<KPublicTransport::Location> dests;
+
     auto prevRes = m_resMgr->reservation(m_batchId);
     auto prevBatchId = m_batchId;
+    if (!isJourneyCandidate(prevRes)) {
+        return {};
+    }
+    dests.push_back(PublicTransport::locationFromPlace(LocationUtil::arrivalLocation(prevRes), prevRes));
     while (true) {
         auto endBatchId = m_resMgr->nextBatch(prevBatchId);
         auto endRes = m_resMgr->reservation(endBatchId);
@@ -494,15 +510,38 @@ KPublicTransport::JourneyRequest TimelineDelegateController::journeyRequestFull(
             break;
         }
 
-        req.setTo(PublicTransport::locationFromPlace(LocationUtil::arrivalLocation(endRes), endRes));
+        dests.push_back(PublicTransport::locationFromPlace(LocationUtil::arrivalLocation(endRes), endRes));
         prevRes = endRes;
         prevBatchId = endBatchId;
     }
 
-    return req;
+    return dests;
 }
 
-void TimelineDelegateController::applyJourney(const QVariant &journey, bool includeFollowing)
+static QVariant applyReservationProperties(QVariant to, const QVariant &res)
+{
+    if (!JsonLd::canConvert<Reservation>(res)) {
+        return to;
+    }
+
+    const auto mo = Reservation::staticMetaObject;
+    for (auto i = 0; i < mo.propertyCount(); ++i) {
+        const auto prop = mo.property(i);
+        if (!prop.isWritable() || !prop.isStored() || std::strcmp(prop.name(), "reservationFor") == 0) {
+            continue;
+        }
+
+        const auto v = prop.readOnGadget(res.constData());
+        if (v.isNull()) {
+            continue;
+        }
+        prop.writeOnGadget(to.data(), v);
+    }
+
+    return to;
+}
+
+void TimelineDelegateController::applyJourney(const QVariant &journey, int includeFollowing)
 {
     if (!m_resMgr || !m_tripGroupMgr || m_batchId.isEmpty()) {
         return;
@@ -519,21 +558,10 @@ void TimelineDelegateController::applyJourney(const QVariant &journey, bool incl
 
     // find all batches we are replying here (same logic as in journeyRequest)
     std::vector<QString> oldBatches({m_batchId});
-    if (includeFollowing) {
-        auto prevRes = m_resMgr->reservation(m_batchId);
-        auto prevBatchId = m_batchId;
-        while (true) {
-            auto endBatchId = m_resMgr->nextBatch(prevBatchId);
-            auto endRes = m_resMgr->reservation(endBatchId);
-            qDebug() << endRes << isJourneyCandidate(endRes) << isLayover(prevRes, endRes);
-            if (!isJourneyCandidate(endRes) || !isLayover(prevRes, endRes)) {
-                break;
-            }
-
-            oldBatches.push_back(endBatchId);
-            prevRes = endRes;
-            prevBatchId = endBatchId;
-        }
+    auto batchId = m_batchId;
+    for (auto i = 0; i < includeFollowing; ++i) {
+        batchId = m_resMgr->nextBatch(batchId);
+        oldBatches.push_back(batchId);
     }
     qCDebug(Log) << "Affected batches:" << oldBatches;
 
@@ -569,9 +597,8 @@ void TimelineDelegateController::applyJourney(const QVariant &journey, bool incl
             // copy ticket data from previous element
             // TODO this would need to be done for the entire batch!
             if (!lastResId.isEmpty()) {
-                auto lastRes = m_resMgr->reservation(lastResId);
-                JsonLdDocument::writeProperty(lastRes, "reservationFor", {});
-                res = JsonLdDocument::apply(lastRes, res);
+                const auto lastRes = m_resMgr->reservation(lastResId);
+                res = applyReservationProperties(res, lastRes);
             }
 
             const auto resId = m_resMgr->addReservation(res);
