@@ -17,13 +17,16 @@
 #include <QAbstractItemModel>
 #include <QColor>
 #include <QDateTime>
+#include <QDir>
 #include <QFile>
 #include <QImage>
+#include <QStandardPaths>
 #include <QTimeZone>
 #include <QUrl>
 
 #include <cmath>
 
+using namespace Qt::Literals;
 using namespace KItinerary;
 
 QDateTime Util::dateTimeStripTimezone(const QVariant &obj, const QString &propertyName)
@@ -127,4 +130,34 @@ QString Util::toBase64(const QByteArray &data)
     return QString::fromLatin1(data.toBase64());
 }
 
+QString Util::cacheLocation(const QString &relativePath)
+{
+    auto path = QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation) + '/'_L1 + relativePath;
+    QDir().mkpath(path);
+    return path;
+}
+
+QString Util::slugify(const QString &input)
+{
+    QString s;
+    s.reserve(input.size());
+    for (auto c : input) {
+        if (c.isLetter() || c.isDigit()) {
+            c = c.toCaseFolded();
+            if (c.decompositionTag() == QChar::Canonical) {
+                c = c.decomposition().at(0);
+            }
+            s.push_back(c);
+        } else if (!s.isEmpty() && s.back() != '-'_L1) {
+            s.push_back('-'_L1);
+        }
+    }
+
+    if (s.endsWith('-'_L1)) {
+        s.chop(1);
+    }
+
+    return s;
+}
+
 #include "moc_util.cpp"
